@@ -27,7 +27,7 @@ compute_distances(EigenPointCloudConstRef corepoints,
   uncertainties.resize(corepoints.rows());
 
 #ifdef PY4DGEO_WITH_OPENMP
-#pragma omp parallel for
+#pragma omp parallel for schedule(dynamic, 1)
 #endif
   for (IndexType i = 0; i < corepoints.rows(); ++i) {
     // Either choose the ith row or the first (if there is no per-corepoint
@@ -40,7 +40,7 @@ compute_distances(EigenPointCloudConstRef corepoints,
       epoch2, scale, corepoints.row(i), dir, max_cylinder_length, i);
 
     // Distance calculation
-    distances[i] = dir.dot(subset1.colwise().mean() - subset2.colwise().mean());
+    distances[i] = dir.dot(subset2.colwise().mean() - subset1.colwise().mean());
 
     // Uncertainty calculation
     uncertainties[i] = uncertaintycalculator(subset1, subset2, dir);
@@ -69,31 +69,56 @@ cylinder_workingset_finder(const Epoch& epoch,
                            double max_cylinder_length,
                            IndexType core_idx)
 {
-  // The search radius is the maximum of cylinder length and radius
-  auto search_radius = std::max(radius, max_cylinder_length);
+  // Cut the cylinder into N segments, perform radius searches around the
+  // segment midpoints and create the union of indices. Afterwards, select
+  // only those points that are within the cylinder
 
-  // Find the points in the radius of max_cylinder_length
-  KDTree::RadiusSearchResult ball_points;
-  epoch.kdtree.precomputed_radius_search(core_idx, search_radius, ball_points);
-  auto superset = epoch.cloud(ball_points, Eigen::all);
+  // The number of segments - later cast to int
+  double N = 1.0;
+  if (max_cylinder_length > radius)
+    N = std::ceil(max_cylinder_length / radius);
+  else
+    max_cylinder_length = radius;
 
-  // If max_cylinder_length is sufficiently small, we are done
-  if (max_cylinder_length <= radius)
-    return superset;
+  // The search radius for each segment
+  double r_cyl = std::sqrt(radius * radius +
+                           max_cylinder_length * max_cylinder_length / (N * N));
 
-  // Calculate the squared distances to the cylinder axis
-  auto distances = (superset.rowwise() - corepoint.row(0))
-                     .rowwise()
-                     .cross(direction.row(0))
-                     .rowwise()
-                     .squaredNorm()
-                     .eval();
+  // Perform radius searches and merge results
+  std::vector<IndexType> merged;
+  for (std::size_t i = 0; i < static_cast<std::size_t>(N); ++i) {
+    auto qp = (corepoint.row(0) +
+               ((2 * i + 1 - N) / N) * max_cylinder_length * direction.row(0))
+                .eval();
+    KDTree::RadiusSearchResult ball_points;
+    epoch.kdtree.radius_search(&(qp(0, 0)), r_cyl, ball_points);
+    merged.reserve(merged.capacity() + ball_points.size());
+    std::copy(
+      ball_points.begin(), ball_points.end(), std::back_inserter(merged));
+  }
+
+  // Making indices unique
+  std::sort(merged.begin(), merged.end());
+  merged.erase(std::unique(merged.begin(), merged.end()), merged.end());
+
+  // Extracting points
+  auto superset = epoch.cloud(merged, Eigen::all);
+
+  // Calculate the squared distances to the cylinder axis and to the plane
+  // perpendicular to the axis that contains the corepoint
+  auto to_corepoint = (superset.rowwise() - corepoint.row(0)).eval();
+  auto to_corepoint_plane = (to_corepoint * direction.transpose()).eval();
+  auto to_axis2 = (to_corepoint - to_corepoint_plane * direction)
+                    .rowwise()
+                    .squaredNorm()
+                    .eval();
 
   // Non-performance oriented version of index extraction. There should
   // be a version using Eigen masks, but I could not find it.
   std::vector<Eigen::Index> indices;
   for (Eigen::Index i = 0; i < superset.rows(); ++i)
-    if (distances(i, 0) < radius * radius)
+    if ((to_axis2(i) <= radius * radius) &&
+        (std::abs(to_corepoint_plane(i)) <= max_cylinder_length))
       indices.push_back(i);
 
   // Select only those indices that are within the cylinder
