@@ -91,7 +91,7 @@ class M3C2LikeAlgorithm(abc.ABC):
 
     def callback_workingset_finder(self):
         """The callback used to determine the point cloud subset around a corepoint"""
-        return _py4dgeo.radius_workingset_finder
+        return _py4dgeo.cylinder_workingset_finder
 
     def callback_uncertainty_calculation(self):
         """The callback used to calculate the uncertainty"""
@@ -123,7 +123,9 @@ class M3C2(M3C2LikeAlgorithm):
 
         # Trigger the precomputation
         if self._precomputed_normals is None:
-            self._precomputed_normals = np.empty(self.corepoints.shape)
+            self._precomputed_normals = np.empty(
+                self.corepoints.shape, dtype=np.float64
+            )
 
             # Find the correct epoch to use for normal calculation
             normals_epoch = self.cloud_for_normals
